/*
** Module   : DropDown.java
** Abstract : Drop down box for combo-box implementation.
**
** Copyright (c) 2010-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SIY 20100804 Created initial version
** 002 SIY 20100822 Refactored AttributeContainer support.
** 003 SIY 20100915 locateFrame() is moved to UiUtils.
** 004 SIY 20111101 Updated imports.
** 005 SIY 20111202 Refactored and moved from CHUI package.
** 006 SIY 20111221 Switched to widget factory.
** 007 CA  20140805 Added generics to force the abstract widget classes to not use implementation-
**                  specific code: the abstract widgets will have access only to the base APIs
**                  provided by the OutputManager.
** 008 HC  20140911 GUI coordinate support - data types refactoring.
** 009 CA  20140926 Added shared widget configuration support, to allow GUI/ChUI concrete
**                  implementation for the same widget ID.  Refactored the widget configuration
**                  classes: all fields were made public; these classes need to be as dumb as
**                  possible, all logic related to setting/getting a certain field should be at
**                  the widget or at the caller. Refs #2254
** 010 HC  20141021 Implemented window move/position support.
** 011 CA  20141119 Refactored drawing code to enable multiple OS windows, for GUI support.  Misc
**                  changes/fixes related to drawing on multiple windows.
** 012 CA  20150212 Allow events to be captured and processed by the widgets without server-side
**                  or main event-loop notification.  This fixes problems in the EDITING and
**                  WAIT-FOR event loops, for i.e. COMBO-BOX and EDITOR widgets.
** 013 IAS 20150214 Added getSelectedIndex() method
** 014 HC  20150323 Removed business logic from config classes.
** 015 EVL 20150427 Separation GUI related code in standalone class.  Making common methods to be
**                  public.
** 016 EVL 20150514 Adding method to adjust the visible rows value according to UI type used.
*/

package com.goldencode.p2j.ui.client;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.border.*;
import com.goldencode.p2j.ui.client.model.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * Implements the drop-down window for a combo-box.
 * TODO<SIY>: this class is not managed by layout manager, review it and make
 * sure that it conforms GUI requirement, i.e. properly maintains its physical 
 * position/size. 
 */
public abstract class DropDown<O extends OutputManager<?>>
extends OuterFrame<O>
{
   /** The list instance to represent the drop-down content. */
   private final ScrollableList<O> list;

   /** Wraps the list of items and handles scrolling. */
   protected ScrollPane<O> scrollpane;

   /** Reference to parent combo-box. */
   private final ComboBox<O> comboBox;

   /**
    * Create a drop-down container based on the parent combo-box and it's
    * associated data model.
    * 
    * @param    parent
    *           The combo-box object which calls this drop-down.
    */
   @SuppressWarnings("unchecked")
   public DropDown(ComboBox<O> parent)
   {
      super("");
      this.comboBox = parent;
      list = screen().getFactory().createScrollableList(parent, parent.model());

      // enable scrolling, ensure there is no extra border or spacing
      scrollpane = screen().getFactory().createScrollPane(WidgetId.nextID(), list);
      scrollpane.setBorder(new EmptyBorder<O>());
      super.add(scrollpane);

      // set height, width and position of the drop-down itself
      list.setColumns(parent.calcLimit(false));
      setHeightAndPos(parent, parent.model());
      
      // force the initial selection
      forceInitialSelection(parent.model());
      // set up notifications
      list.addSelectionListener(parent);

      OutputManager.instance().getRegistry().addWidget((Widget) scrollpane);
   }

   /**
    * Get the combo-box visible row adjust depending on the current UI type in use.
    * 
    * @param    visibleRows
    *           The visible rows value to check and modify.
    * @param    listSize
    *           The total size of the items in the list to handle.
    * 
    * @return  the new visible rows value.
    */
   protected abstract int fixVisibleRows(int visibleRows, int listSize);

   /**
    * Enforce the current selection.
    *
    * @param    model
    *           The data model to be used within this drop-down list.
    */
   public void forceInitialSelection(ComboBoxModel<String> model) 
   {
      int idx = model.calcSelection(comboBox.config().items);
      
      list.select(idx);
      // set up initial top row
      list.setTopRow(idx);
   }
   
   /**
    * Get the widget associated with the drop-down's inner list.
    * 
    * @return   The {@link #scrollpane}.
    */
   public Widget<O> getList()
   {
      return scrollpane;
   }
   
   
   /**
    * Get the selected object from the drop-down list.
    * 
    * @return   The object which is currently selected in drop-down list
    *           or <code>null</code> if there is no selection.
    */
   public String getSelectedItem()
   {
      return list.value();
   }
   
   /**
    * Get the index of the selected object from the drop-down list.
    * 
    * @return   The index of the object which is currently selected in drop-down list
    */
   public int getSelectedIndex() 
   {
      return list.index();
   }
   
   /**
    * Calculate (and set) the height and position of the drop-down
    * container based on the parent combo-box and it's associated data 
    * model.
    * <p>
    * Precedence order (in the following list the term "max-space" is
    * the number of rows from the top of the containing combo-box's
    * frame to the bottom of the screen) for vertical size:
    * <p>
    * <pre>
    * 1. If there is an inner-lines definition AND it is smaller than the 
    *    max-space, then it is honored.
    * 2. If the number of elements in the data model is less than the
    *    number of rows in the max-space, then the number of data model
    *    elements will determine the vertical size.
    * 3. max-space is used otherwise.
    * </pre>
    * <p>
    * The drop-down box X coordinate is always the same as the X
    * coordinate of the parent combo-box in absolute screen units. For
    * the Y coordinate, the following positioning algorithm is used:
    * <p>
    * <pre>
    * 1. If the height of the drop-down is less than or equal to the space
    *    between the bottom of the combo-box and the bottom of the screen,
    *    then the drop-down T coordinate will be set to the combo-box Y +
    *    1.
    * 2. Otherwise, the drop-down Y coordinate will equal the Y coordinate
    *    of the bottom of the screen minus the height of the drop-down.
    *    This will result in the drop-down ending at the bottom of the
    *    screen and starting at or above the combo-box Y coordinate BUT
    *    never above the top of the frame containing the combo-box
    *    (because of the previously constrained height calculation above).
    * </pre>
    * 
    * @param    parent
    *           The combo-box object which calls this drop-down.
    * @param    model
    *           The data model to be used within this drop-down list.
    */
   public void setHeightAndPos(ComboBox<O> parent, ComboBoxModel<String> model)
   {
      Dimension screenSize = parent.screenDimension();

      @SuppressWarnings("unchecked")
      Frame<O> frame = (Frame<O>) UiUtils.locateFrame(parent);
      
      // calculate the maximum size available for the drop-down (the space
      // from the row just below the top of the containing frame to the 
      // bottom of the screen reserving 2 lines for the drop-down top and
      // bottom border)
      // TODO: for some reason we don't use value set by 
      //       ComboBoxConfig.setMaximumRowCount()!!!
      Point frameLoc = frame.screenLocation();
      double maxSize  = screenSize.height - frameLoc.y - 1 - 2;
      
      ComboBoxConfig config = parent.config();
      
      // query inner lines (explicit size request)
      int rows = config.innerLines;
      
      // override inner lines with the number of elements if not specified
      if (rows <= 0)
      {
         int numItems = model.size();
         
         // ensure that we always have at least 1 row
         rows = (numItems > 0) ? numItems : 1;
      }
      
      // if the inner lines spec or the number of elements are more than
      // the maximum space, reset to the maximum size
      if (rows > (int) maxSize)
      {
         rows = (int) maxSize;
      }
      
      // differnt UI types take differen approach here
      rows = fixVisibleRows(rows, model.size());
      
      int frameHeight = rows + 2;
      
      // set the frame height
      setHeight(frameHeight);
      
      // set the list height
      list.setVisibleRows(rows);
      
      setSize(minimumSize());
      doLayout();
      
      // get the absolute origin of the parent combo-box
      Point origin = parent.screenLocation();
      
      // calc default Y coordinate for the drop-down
      double y = origin.y + 1; 
      
      // do we have enough room below the combo-box to hold the entire
      // drop-down?
      if (Coordinate.scale(screenSize.height - y) >= frameHeight)
      {
         // yes, use the default
         origin.y = y;
      }
      else
      {
         // no, find the location above the default where we must position
         // the drop-down
         origin.y = screenSize.height - frameHeight;
      }
      
      //TODO<SIY>: set physical position too 
      // set the drop-down position
      setLocation(origin);
   }
   
   /**
    * Get the combo-box widget to which this drop-down belongs.
    * 
    * @return  the combo-box widget to which this drop-down is attached.
    */
   public ComboBox<O> getComboBox()
   {
      return comboBox;
   }

   /**
    * Hide this widget and remove it.
    */
   @Override
   public void hide()
   {
      super.hide();
      
      // this is a special case: if the drop-down is hidden, then its no longer available.
      WindowManager.remove(this);
   }
}
